==> icloudpd-1.17.6-py3-none-any.whl: icloudpd/authentication.py <==
"""Handles username/password authentication and two-step authentication"""

import logging
import sys
from typing import Callable, Optional
import click
import pyicloud_ipd


class TwoStepAuthRequiredError(Exception):
    """
    Raised when 2SA is required. base.py catches this exception
    and sends an email notification.
    """


def authenticator(logger: logging.Logger, domain: str) -> Callable[[str, Optional[str], Optional[str], bool, Optional[str]], pyicloud_ipd.PyiCloudService]:
    """Wraping authentication with domain context"""
    def authenticate_(
            username:str,
            password:Optional[str],
            cookie_directory:Optional[str]=None,
            raise_error_on_2sa:bool=False,
            client_id:Optional[str]=None,
    ) -> pyicloud_ipd.PyiCloudService:
        """Authenticate with iCloud username and password"""
        logger.debug("Authenticating...")
        while True:
            try:
                # If password not provided on command line variable will be set to None
                # and PyiCloud will attempt to retrieve from its keyring
                icloud = pyicloud_ipd.PyiCloudService(
                    domain,
                    username, password,
                    cookie_directory=cookie_directory,
                    client_id=client_id,
                )
                break
            except pyicloud_ipd.exceptions.PyiCloudNoStoredPasswordAvailableException:
                # Prompt for password if not stored in PyiCloud's keyring
                password = click.prompt("iCloud Password", hide_input=True)

        if icloud.requires_2fa:
            if raise_error_on_2sa:
                raise TwoStepAuthRequiredError(
                    "Two-step/two-factor authentication is required"
                )
            logger.info("Two-step/two-factor authentication is required (2fa)")
            request_2fa(icloud, logger)

        elif icloud.requires_2sa:
            if raise_error_on_2sa:
                raise TwoStepAuthRequiredError(
                    "Two-step/two-factor authentication is required"
                )
            logger.info("Two-step/two-factor authentication is required (2sa)")
            request_2sa(icloud, logger)

        return icloud
    return authenticate_


def request_2sa(icloud: pyicloud_ipd.PyiCloudService, logger: logging.Logger) -> None:
    """Request two-step authentication. Prompts for SMS or device"""
    devices = icloud.trusted_devices
    devices_count = len(devices)
    device_index = 0
    if devices_count > 0:
        for i, device in enumerate(devices):
            # pylint: disable-msg=consider-using-f-string
            print(
                "  %s: %s" %
                (i, device.get(
                    "deviceName", "SMS to %s" %
                    device.get("phoneNumber"))))
            # pylint: enable-msg=consider-using-f-string

        device_index = click.prompt(
            "Please choose an option:",
            default=0,
            type=click.IntRange(
                0,
                devices_count - 1))

    device = devices[device_index]
    if not icloud.send_verification_code(device):
        logger.error("Failed to send two-factor authentication code")
        sys.exit(1)

    code = click.prompt("Please enter two-factor authentication code")
    if not icloud.validate_verification_code(device, code):
        logger.error("Failed to verify two-factor authentication code")
        sys.exit(1)
    logger.info(
        "Great, you're all set up. The script can now be run without "
        "user interaction until 2SA expires.\n"
        "You can set up email notifications for when "
        "the two-step authentication expires.\n"
        "(Use --help to view information about SMTP options.)"
    )


def request_2fa(icloud: pyicloud_ipd.PyiCloudService, logger: logging.Logger) -> None:
    """Request two-factor authentication."""
    try:
        devices = icloud.trusted_devices
    except:
        devices = []
    if len(devices) > 0:
        if len(devices) > 99:
            logger.error("Too many trusted devices for authentication")
            sys.exit(1)

        for i, device in enumerate(devices):
            # pylint: disable-msg=consider-using-f-string
            print(
                "  %s: %s" %
                (i, device.get(
                    "deviceName", "SMS to %s" %
                    device.get("phoneNumber"))))
            # pylint: enable-msg=consider-using-f-string

        index_str = f"..{len(devices) - 1}" if len(devices) > 1 else ""
        code = click.prompt(
            f"Please enter two-factor authentication code or device index (0{index_str}) to send SMS with a code",
            type=click.IntRange(
                0,
                999999))

        if code < 100:
            # need to send code
            device = devices[code]
            if not icloud.send_verification_code(device):
                logger.error("Failed to send two-factor authentication code")
                sys.exit(1)
            code = click.prompt(
                "Please enter two-factor authentication code that you received over SMS",
                type=click.IntRange(
                    0,
                    999999))
            if not icloud.validate_verification_code(device, code):
                logger.error("Failed to verify two-factor authentication code")
                sys.exit(1)
        else:
            if not icloud.validate_2fa_code(code):
                logger.error("Failed to verify two-factor authentication code")
                sys.exit(1)
    else:
        code = click.prompt(
            "Please enter two-factor authentication code",
            type=click.IntRange(
                0,
                999999))
        if not icloud.validate_2fa_code(code):
            logger.error("Failed to verify two-factor authentication code")
            sys.exit(1)
    logger.info(
        "Great, you're all set up. The script can now be run without "
        "user interaction until 2SA expires.\n"
        "You can set up email notifications for when "
        "the two-step authentication expires.\n"
        "(Use --help to view information about SMTP options.)"
    )

==> icloudpd-1.17.7-py3-none-any.whl: icloudpd/authentication.py <==
"""Handles username/password authentication and two-step authentication"""

import logging
import sys
from typing import Callable, Optional
import click
import pyicloud_ipd
from pyicloud_ipd.base import PyiCloudService


class TwoStepAuthRequiredError(Exception):
    """
    Raised when 2SA is required. base.py catches this exception
    and sends an email notification.
    """


def authenticator(logger: logging.Logger, domain: str) -> Callable[[str, Optional[str], Optional[str], bool, Optional[str]], PyiCloudService]:
    """Wraping authentication with domain context"""
    def authenticate_(
            username:str,
            password:Optional[str],
            cookie_directory:Optional[str]=None,
            raise_error_on_2sa:bool=False,
            client_id:Optional[str]=None,
    ) -> PyiCloudService:
        """Authenticate with iCloud username and password"""
        logger.debug("Authenticating...")
        while True:
            try:
                # If password not provided on command line variable will be set to None
                # and PyiCloud will attempt to retrieve from its keyring
                icloud = PyiCloudService(
                    domain,
                    username, password,
                    cookie_directory=cookie_directory,
                    client_id=client_id,
                )
                break
            except pyicloud_ipd.exceptions.PyiCloudNoStoredPasswordAvailableException:
                # Prompt for password if not stored in PyiCloud's keyring
                password = click.prompt("iCloud Password", hide_input=True)

        if icloud.requires_2fa:
            if raise_error_on_2sa:
                raise TwoStepAuthRequiredError(
                    "Two-step/two-factor authentication is required"
                )
            logger.info("Two-step/two-factor authentication is required (2fa)")
            request_2fa(icloud, logger)

        elif icloud.requires_2sa:
            if raise_error_on_2sa:
                raise TwoStepAuthRequiredError(
                    "Two-step/two-factor authentication is required"
                )
            logger.info("Two-step/two-factor authentication is required (2sa)")
            request_2sa(icloud, logger)

        return icloud
    return authenticate_


def request_2sa(icloud: PyiCloudService, logger: logging.Logger) -> None:
    """Request two-step authentication. Prompts for SMS or device"""
    devices = icloud.trusted_devices
    devices_count = len(devices)
    device_index = 0
    if devices_count > 0:
        for i, device in enumerate(devices):
            # pylint: disable-msg=consider-using-f-string
            print(
                "  %s: %s" %
                (i, device.get(
                    "deviceName", "SMS to %s" %
                    device.get("phoneNumber"))))
            # pylint: enable-msg=consider-using-f-string

        device_index = click.prompt(
            "Please choose an option:",
            default=0,
            type=click.IntRange(
                0,
                devices_count - 1))

    device = devices[device_index]
    if not icloud.send_verification_code(device):
        logger.error("Failed to send two-factor authentication code")
        sys.exit(1)

    code = click.prompt("Please enter two-factor authentication code")
    if not icloud.validate_verification_code(device, code):
        logger.error("Failed to verify two-factor authentication code")
        sys.exit(1)
    logger.info(
        "Great, you're all set up. The script can now be run without "
        "user interaction until 2SA expires.\n"
        "You can set up email notifications for when "
        "the two-step authentication expires.\n"
        "(Use --help to view information about SMTP options.)"
    )


def request_2fa(icloud: PyiCloudService, logger: logging.Logger) -> None:
    """Request two-factor authentication."""
    try:
        devices = icloud.trusted_devices
    except:
        devices = []
    if len(devices) > 0:
        if len(devices) > 99:
            logger.error("Too many trusted devices for authentication")
            sys.exit(1)

        for i, device in enumerate(devices):
            # pylint: disable-msg=consider-using-f-string
            print(
                "  %s: %s" %
                (i, device.get(
                    "deviceName", "SMS to %s" %
                    device.get("phoneNumber"))))
            # pylint: enable-msg=consider-using-f-string

        index_str = f"..{len(devices) - 1}" if len(devices) > 1 else ""
        code = click.prompt(
            f"Please enter two-factor authentication code or device index (0{index_str}) to send SMS with a code",
            type=click.IntRange(
                0,
                999999))

        if code < 100:
            # need to send code
            device = devices[code]
            if not icloud.send_verification_code(device):
                logger.error("Failed to send two-factor authentication code")
                sys.exit(1)
            code = click.prompt(
                "Please enter two-factor authentication code that you received over SMS",
                type=click.IntRange(
                    0,
                    999999))
            if not icloud.validate_verification_code(device, code):
                logger.error("Failed to verify two-factor authentication code")
                sys.exit(1)
        else:
            if not icloud.validate_2fa_code(code):
                logger.error("Failed to verify two-factor authentication code")
                sys.exit(1)
    else:
        code = click.prompt(
            "Please enter two-factor authentication code",
            type=click.IntRange(
                0,
                999999))
        if not icloud.validate_2fa_code(code):
            logger.error("Failed to verify two-factor authentication code")
            sys.exit(1)
    logger.info(
        "Great, you're all set up. The script can now be run without "
        "user interaction until 2SA expires.\n"
        "You can set up email notifications for when "
        "the two-step authentication expires.\n"
        "(Use --help to view information about SMTP options.)"
    )
